/********************************************************************
TCP/IP Server Application
File Name: main.cpp

DESCRIPTION
This Win32 console application is a TCP/IP server that will listen 
for connections on the specified port number.  The application
is written for Winsock 1.1 compliance.


COMMAND LINE PARAMETERS



COMPILILATION INSTRUCTIONS



REVISION HISTORY
12/22/2000	Initial Release


*********************************************************************/

#include <winsock.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <memory.h>

#define RX_BUFSIZE	(10000)

//----- functions -----
void ProcessCmdLine(int argc, char **argv);
DWORD WINAPI ProcessClient(LPVOID lpParam);


//----- global vars -----
WORD dListenPort;
char szIPAddress[80];	
BOOL bEcho = FALSE;
DWORD dwStartTick, dwEndTick;
DWORD dwTotalBytesRead;

char RxBuf[RX_BUFSIZE];


/*-------------------------------------------------------------------
 FUNCTION: ProcessCmdLine

 DESCRIPTION:
 Process command line arguments

 RETURNS: Nothing
 ------------------------------------------------------------------*/
void ProcessCmdLine(int argc, char **argv)
{
	if (argc < 2)
	{
		printf("Usage: TCPServer <port> <echo>\n");
		exit(1);
	}

	dListenPort = (WORD) atoi(argv[1]);

	if (argc > 2)
	{
		if (strcmp(argv[2], "echo") == 0)
			bEcho = TRUE;
	}
}



/*-------------------------------------------------------------------
 FUNCTION: ProcessClient

 DESCRIPTION:
 This function is called as a thread to handle client connections.

 PARAMETERS
 lpParam is a pointer to the client socket handle.

 RETURNS: 32-bit thread exit code
 ------------------------------------------------------------------*/
DWORD WINAPI ProcessClient(LPVOID lpParam)
{
	SOCKET s = (SOCKET)lpParam;
	int SampleCtr = 0;

	dwTotalBytesRead = 0;
	while (1)
	{
		int bytes_read = recv(s, RxBuf, RX_BUFSIZE, 0);
		RxBuf[bytes_read] = '\0';
		printf("Received %d bytes: %s\n", bytes_read, RxBuf);
		dwTotalBytesRead += bytes_read;

		if ( (bytes_read > 0) && bEcho )
		{
			if (send(s, RxBuf, strlen(RxBuf), 0) == SOCKET_ERROR)
			{
				printf("Error sending echo data: %d\n", WSAGetLastError());
			}
		}
		else if (bytes_read == 0)
		{
			closesocket(s);
			//exit(0);
			break;
		}
		else if (bytes_read == SOCKET_ERROR)
		{
			printf("recv() error in ProcessClient(): %d\n", WSAGetLastError());
			break;
		}
	}
	return 0;
}





/*-------------------------------------------------------------------
 FUNCTION: main

 DESCRIPTION:
 Main routine to start network services and listen for connections.

 RETURNS: integer exit code
 ------------------------------------------------------------------*/
int main(int argc, char **argv)
{
	SOCKET	sListen, sClient;
	WSADATA	WSAData;
	struct sockaddr_in sinListen, sinClient;


	ProcessCmdLine(argc, argv);
	
	if (WSAStartup( MAKEWORD(1,1), &WSAData) != 0)
	{
		printf("Error loading Winsock\n");
		return 1;
	}
	
	// Create socket to listen on
	sListen = socket(AF_INET, SOCK_STREAM, 0);
	if (sListen == SOCKET_ERROR)
	{
		printf("Error creating sListen socket: %d\n", WSAGetLastError());
		return 1;
	}

	// Setup listen address structure
	sinListen.sin_addr.s_addr = htonl(INADDR_ANY);
	sinListen.sin_family = AF_INET;
	sinListen.sin_port = htons(dListenPort);

	// Bind socket to listen address
	if (bind(sListen, (struct sockaddr *) &sinListen, sizeof(sinListen)) == SOCKET_ERROR)
	{
		printf("bind() error for sListen: %d\n", WSAGetLastError());
		return 1;
	}

	// Listen to interface and port
	if (listen(sListen, 5) == SOCKET_ERROR)
	{
		printf("listen() error for sListen: %d\n", WSAGetLastError());
		return 1;
	}

	while (1)
	{
		printf("Listening on port: %d....\n", dListenPort);
		int len = sizeof(sinClient);
		sClient = accept(sListen, (struct sockaddr *) &sinClient, &len);
		if (sClient == INVALID_SOCKET)
		{
			printf("accept() error: %d\n", WSAGetLastError());
			break;
		}

		printf("Accepted connection from: %s:%d\n",
			inet_ntoa(sinClient.sin_addr), ntohs(sinClient.sin_port));



		/*
		HANDLE CreateThread( 
				LPSECURITY_ATTRIBUTES lpThreadAttributes, // pointer to thread security attributes 
				DWORD dwStackSize,						// initial thread stack size, in bytes 
 				LPTHREAD_START_ROUTINE lpStartAddress,  // pointer to thread function 
				LPVOID lpParameter,						// argument for new thread 
				DWORD dwCreationFlags,					// creation flags 
				LPDWORD lpThreadId						// pointer to returned thread identifier 
				); 
		*/
		HANDLE hThread;
		DWORD  dwThreadId;
		hThread = CreateThread(NULL, 0, ProcessClient, (LPVOID)sClient, 0, &dwThreadId);
		if (hThread == NULL)
		{
			printf("Error creating Client thread: %d\n", GetLastError());
			break;
		}

	}

	closesocket(sListen);
	WSACleanup();
	return 0;
}







